package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.action;


import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.IdentityCardService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 *
 * @author lixinxu
 *
 */
@Slf4j
@RestController
public class IdentityCardAction {
	/**日志参数中的错误参数名*/
	private static final String LOG_ERROR_PARA = "x-responseCode";
	/**日志参数中的是否收费*/
	private static final String LOG_FEE_PARA = "x-isCharge";

	@Autowired
	private IdentityCardService service;


	@ApiOperation(value = "身份证二要素验证", notes = "")
	@PostMapping(value = "/identityCardTwoPointCheck.do")
	public ChannelTypeHandleResponseObject identityCardTwoPointCheck(
			@ApiParam(value="通道ID")@RequestHeader(name="channelId")  String channelId,
			@ApiParam(value="交科码|卓望码")@RequestHeader(name="ifJkCode")  String ifJkCode,
			@ApiParam(value="超时时间,单位：毫秒")@RequestParam(name = "outTime", required = true) int outTime,
			@ApiParam(value="请求参数JSON串")@RequestParam(name = "customBody", required = true) String customBody) {
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
				.getRequest();

		//调用service
		ChannelTypeHandleResponseObject ret = service.identityCardTwoPointCheck(request, customBody, outTime, ifJkCode);

		//把接口参数、调用结果和是否收费放入访问日志中
		if(ret.getCode() != Constant.SUCCESS) {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_ERROR_CODE);
		} else {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_SUCCESS_CODE);
		}
		request.setAttribute(LOG_FEE_PARA, ret.getIsCharge());

		try {
			request.setAttribute(Constant.CHANNEL_LOG_QUERY,
					Base64.encodeBase64String(customBody.getBytes("UTF-8")));

		} catch (Exception ex) {
			ErrorUtils.captureException(ex);
			log.error("Supplier10000035Action.identityCardTwoPointCheck.Exception{}", ex);
		}

		request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
		return ret;
	}

	@ApiOperation(value = "身份证二要素验证-WeiZhong解密", notes = "")
	@PostMapping(value = "/identityCardTwoWeiZhongDecode.do")
	public ChannelTypeHandleResponseObject identityCardTwoWeiZhongDecode(
			@ApiParam(value="通道ID")@RequestHeader(name="channelId")  String channelId,
			@ApiParam(value="交科码|卓望码")@RequestHeader(name="ifJkCode")  String ifJkCode,
			@ApiParam(value="超时时间,单位：毫秒")@RequestParam(name = "outTime", required = true) int outTime,
			@ApiParam(value="请求参数JSON串")@RequestParam(name = "customBody", required = true) String customBody) {
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
				.getRequest();

		//调用service
		ChannelTypeHandleResponseObject ret = service.twoElementsCheckWeiZhongDecode(request, customBody, outTime, ifJkCode);

		//把接口参数、调用结果和是否收费放入访问日志中
		if(ret.getCode() != Constant.SUCCESS) {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_ERROR_CODE);
		} else {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_SUCCESS_CODE);
		}
		request.setAttribute(LOG_FEE_PARA, ret.getIsCharge());

		try {
			request.setAttribute(Constant.CHANNEL_LOG_QUERY,
					Base64.encodeBase64String(customBody.getBytes("UTF-8")));

		} catch (Exception ex) {
			ErrorUtils.captureException(ex);
			log.error("Supplier10000003Action.twoElementsCheckWeiZhongDecode.Exception{}", ex);
		}

		request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
		return ret;
	}
}
